function example_testbench_60Hz
% example_testbench_60Hz - Flickermeter Simulator Performance Test
%
% Performs the performance tests according [1], section 6.3 for a
% 60 Hz system.
%
% Requires MATLAB with Signal Processing Toolbox installed or Octave.
% For more information refer to [2].
%===============================================================================
% References:
% [1] IEC 61000-4-15, Electromagnetic compatibility (EMC), Testing and
%     measurement techniques, Flickermeter, Edition 2.0, 2010-08
% [2] https://www.solcept.ch/en/blog/complex-systems/flickersim/
%===============================================================================
%  (c) Copyright 2018 Solcept AG
%  Distributed under the Boost Software License, Version 1.0. (See accompanying
%  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
%===============================================================================

clear variables

%% Configuration

F_LINE               =   60;  % line frequency in Hz
FS                   = 4000;  % sampling frequency in Hz
OBSERVATION_INTERVAL =  600;  % duration of simulation in sec
ERROR_LIMIT          =    5;  % maximum deviation of Pst in percent

IS_OCTAVE = exist('OCTAVE_VERSION') ~= 0;

%% Definition of test cases (according [1], section 6.3)

% voltage changes per minute
rate = [    1,     2,     7,    39,   110,  1620,  4800];

% voltage of line
v_line = [120, 230];

% magnitude of relative voltage changes in percent
d = [3.181, 2.564, 1.694, 1.040, 0.844, 0.548, 4.837; % 120 V @ 60Hz
     2.719, 2.194, 1.450, 0.895, 0.723, 0.409, 3.263]; % 230 V @ 60Hz

% scaling factors to be tested
scale_factors = [0.1, 0.5, 1, 2, 3, 5];

%% Run Tests
fprintf('\nStarting Flicker Model Test\n\n');

all_tests_passed = 1;
t = 0 : 1 / FS : OBSERVATION_INTERVAL;

for (voltage_idx = 1:size(d, 1))
    fprintf('Testing with voltage: %3d \n\n', v_line(voltage_idx));
    for (s = scale_factors)
        fprintf('Testing with scaling factor: %.1f\n', s);
        for (idx = 1 : length(rate))
            f_mod = rate(idx) / (2 * 60);
            a_mod = s * d(voltage_idx, idx) / 100;
            
            % create test signal (according [1], Annex B)
            u = sin(2 * pi * F_LINE * t) .* (1 + a_mod * 1 / 2 * sign(sin(2 * pi * f_mod * t)));
            
            P_st(idx) = flicker_sim(u, FS, F_LINE, v_line(voltage_idx));
            err(idx) = (P_st(idx) / s - 1) * 100;
            
            fprintf('V_LINE = %3d V, F_LINE = %2d Hz, r = %4d (%5.2f Hz), d = %6.3f %%:  P_st = %1.3f, err = %4.1f %%\n', ...
                v_line(voltage_idx), F_LINE, rate(idx), f_mod, a_mod * 100, P_st(idx), err(idx))
            if (IS_OCTAVE)
                fflush(stdout);
            end
        end
        
        max_err = max(abs(err));
        if (max_err <= ERROR_LIMIT)
            fprintf('Test passed (max. error: %.1f %%)\n\n', max_err);
        else
            fprintf('Test failed (max. error: %.1f %%)\n\n', max_err);
            all_tests_passed = 0;
        end
    end
end

if (all_tests_passed)
    fprintf('ALL TESTS PASSED\n');
end
