function example_testbench_half_wave_control
% example_testbench_half_wave_control - Flickermeter Simulator Performance Test
%
% Performs the performance tests according [1], section 4.6.1
%
% Requires MATLAB with Signal Processing Toolbox installed or Octave.
% For more information refer to [2].
%===============================================================================
% References:
% [1] Wilhelm Mombauer: "Messung von Spannungsschwankungen und Flickern mit
%     dem IEC-Flickermeter", ISBN 3-8007-2525-8, VDE-Verlag 
% [2] https://www.solcept.ch/en/blog/complex-systems/flickersim/
%===============================================================================
%  (c) Copyright 2021 Solcept AG
%  Distributed under the Boost Software License, Version 1.0. (See accompanying
%  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
%===============================================================================

clear variables

%% Configuration

U_LINE = 230;  % line voltage in Volts
F_LINE = 50;   % line frequency in Hz
FS     = 2000; % sampling frequency in Hz
dU     = 0.01; % 1 percent
SWITCHING_SEQUENCE = [1, 0, 0];

PST_EXPECTED = 0.41;  % expected Pst value

%% Run Test
fprintf('\nStarting Flicker Model Test\n\n');
fprintf('Test half wave control example\n');

repeated_switching_sequence = repmat(SWITCHING_SEQUENCE, 1, 100);
repeated_switching_sequence = repeated_switching_sequence(:);

% generate half wave signal with and without load
nof_samples = round(FS / F_LINE / 2);
t = [0 : nof_samples - 1] / FS;
u_off = U_LINE .* sqrt(2) .* sin(2 * pi * F_LINE * t);  % load off
u_on = (1 - dU) * U_LINE .* sqrt(2) .* sin(2 * pi * F_LINE * t);  % load on

% generate line signal signal
numof_half_waves = length(repeated_switching_sequence);
samples_per_half_wave = length(u_off);
u = zeros(1, numof_half_waves * samples_per_half_wave);
u_ideal = zeros(1, numof_half_waves * samples_per_half_wave);

u_idx = 1;
for (idx = 1 : numof_half_waves)
  % determine sign for next have wave
  if (mod(idx, 2) == 1)
    wave_sign = 1;
  else
    wave_sign = -1;
  end
  
  % add half wave
  if (repeated_switching_sequence(idx) == 1)
    u(u_idx : u_idx + samples_per_half_wave - 1) = wave_sign * u_on;
  else
    u(u_idx : u_idx + samples_per_half_wave - 1) = wave_sign * u_off;
  end
  u_ideal(u_idx : u_idx + samples_per_half_wave - 1) = wave_sign * u_off;
  u_idx = u_idx + samples_per_half_wave;
end

Pst = flicker_sim(u, FS, F_LINE, U_LINE);

du = u_ideal - u;
fprintf('Pst_simulated = %.2f\n', Pst);
fprintf('Pst_expected  = %.2f\n', PST_EXPECTED);
