function example_testbench_rectangular_voltage_changes_with_duty_ratio
% example_testbench_120V_60Hz - Flickermeter Simulator Performance Test
%
% Performs the performance tests according [1], section 6.8
%
% Requires MATLAB with Signal Processing Toolbox installed or Octave.
% For more information refer to [2].
%===============================================================================
% References:
% [1] IEC 61000-4-15, Electromagnetic compatibility (EMC), Testing and
%     measurement techniques, Flickermeter, Edition 2.0, 2010-08
% [2] https://www.solcept.ch/en/blog/complex-systems/flickersim/
%===============================================================================
%  (c) Copyright 2018 Solcept AG
%  Distributed under the Boost Software License, Version 1.0. (See accompanying
%  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
%===============================================================================

clear variables

%% Configuration

F_LINE               =   [50, 60]; % line frequencies in Hz
V_LINE               = [230, 120]; % voltages of line
FS                   =       4000; % sampling frequency in Hz
OBSERVATION_INTERVAL =        600; % duration of simulation in sec
ERROR_LIMIT          =          5; % maximum deviation of Pst in percent

IS_OCTAVE = exist('OCTAVE_VERSION') ~= 0;

%% Definition of test cases (according [1], section 6.8)

% magnitude of relative voltage changes in percent
d_v230_f50    = 1.418; % 230 V, 50Hz
d_v230_f60    = 1.480;
d_v120_f60    = 2.126;
d_v120_f50    = 2.017;

d = [d_v230_f50, d_v230_f60;
     d_v120_f50, d_v120_f60];

f_mod = 28;  % 28 Hertz
DUTY_CYCLE = 0.2;

%% Run Tests
fprintf('\nStarting Flicker Model Test\n\n');
fprintf('Test specification for rectangular voltage changes with duty ratio\n');

all_tests_passed = 1;
t = 0 : 1 / FS : OBSERVATION_INTERVAL - 1/FS;
err = zeros(size(d));
Pst = zeros(size(d));

for (v_idx = 1:length(V_LINE))
    for (f_idx = 1:length(F_LINE))
        u_mod = d(v_idx, f_idx) * 1/2 * sign(sin(2 * pi * f_mod * t)) / 100; % rectangular 28Hz
        
        duty_cycle_1min = [ones(1, 60 * FS * DUTY_CYCLE) zeros(1, 60 * FS * (1 - DUTY_CYCLE))];
        duty_cycle = repmat(duty_cycle_1min, 1, OBSERVATION_INTERVAL / 60);
        
        u_mod = u_mod .* duty_cycle;
        
        % create test signal (according [1], Annex B)
        u = sin(2 * pi * F_LINE(f_idx) * t) .* (1 + u_mod);
        
        P_st(v_idx, f_idx) = flicker_sim(u, FS, F_LINE(f_idx), V_LINE(v_idx));
        err(v_idx, f_idx) = (P_st(v_idx, f_idx) - 1) * 100;
        
        fprintf('V_LINE = %2d V, F_LINE = %2d Hz, d = %6.3f %%:  P_st = %1.3f, err = %4.1f %%\n', ...
            V_LINE(v_idx), F_LINE(f_idx), d(v_idx, f_idx), P_st(v_idx, f_idx), err(v_idx, f_idx))
        if (IS_OCTAVE)
            fflush(stdout);
        end
    end
end

max_err = max(abs(err));
if (max_err <= ERROR_LIMIT)
    fprintf('Test passed (max. error: %.1f %%)\n\n', max_err);
else
    fprintf('Test failed (max. error: %.1f %%)\n\n', max_err);
    all_tests_passed = 0;
end


if (all_tests_passed)
    fprintf('ALL TESTS PASSED\n');
end
